@@ -189,7 +189,7 @@ class PKAssetsManager:
     def afterMarketStockDataExists(intraday=False, forceLoad=False):
         exists, cache_file = Archiver.afterMarketStockDataExists(intraday=intraday,
                                                                  forceLoad=forceLoad,
-                                                                 date_suffix=False)
+                                                                 date_suffix=True)
         return exists, cache_file
 
     @Halo(text='', spinner='dots')
@@ -325,7 +325,16 @@ class PKAssetsManager:
         initialLoadCount = len(stockDict)
         leftOutStocks = None
         recentDownloadFromOriginAttempted = False
+        srcFilePath = os.path.join(Archiver.get_user_data_dir(), cache_file)
         isTrading = PKDateUtilities.isTradingTime() and (PKDateUtilities.wasTradedOn() or not PKDateUtilities.isTodayHoliday()[0])
+        if isTrading or not os.path.exists(srcFilePath):
+            try:
+                from pkbrokers.kite.examples.externals import kite_fetch_save_pickle
+                if kite_fetch_save_pickle():
+                    default_logger().info("pkl file update succeeded!")
+            except Exception as e:
+                default_logger().error(f"Error downloading latest file:{e}")
+            isTrading = False
         if userDownloadOption is not None and "B" in userDownloadOption: # Backtests
             isTrading = False
         # Check if NSEI data is requested
@@ -351,7 +360,6 @@ class PKAssetsManager:
         )
         stockDataLoaded = False
         # copyFilePath = os.path.join(Archiver.get_user_data_dir(), f"copy_{cache_file}")
-        srcFilePath = os.path.join(Archiver.get_user_data_dir(), cache_file)
         # if os.path.exists(copyFilePath):
         #     shutil.copy(copyFilePath,srcFilePath) # copy is the saved source of truth
         if os.path.exists(srcFilePath) and not forceRedownload:
