@@ -30,7 +30,7 @@ import pandas as pd
 from PKDevTools.classes.ColorText import colorText
 from PKDevTools.classes.PKDateUtilities import PKDateUtilities
 from PKDevTools.classes.log import default_logger
-from pkscreener.classes import Utility
+from pkscreener.classes import Utility, ConsoleUtility
 from pkscreener.classes.ConfigManager import parser, tools
 
 configManager = tools()
@@ -75,7 +75,7 @@ def backtest(
         return backTestedData
     data = data.head(max(calcPeriods) + 1)
     # Let's check the returns for the given strategy over a period ranging from 1 period to 30 periods.
-    # columns=['Stock', 'Date', 'Volume', 'Trend', 'MA-Signal', 'LTP', '52Wk-H',
+    # columns=['Stock', 'Date', "volume", 'Trend', 'MA-Signal', 'LTP', '52Wk-H',
     #          '52Wk-L', '1-Pd', '2-Pd', '3-Pd', '4-Pd', '5-Pd', '10-Pd', '15-Pd',
     #          '22-Pd', '30-Pd', 'Consol.', 'Breakout', 'RSI', 'Pattern', 'CCI',
     #          'LTP1', 'Growth1', 'LTP2', 'Growth2', 'LTP3', 'Growth3', 'LTP4',
@@ -96,7 +96,7 @@ def backtest(
     columns=[
                 "Stock",
                 "Date",
-                "Volume",
+                "volume",
                 "Trend",
                 "MA-Signal",
                 "LTP",
@@ -106,7 +106,7 @@ def backtest(
     backTestedStock = {
         "Stock": "",
         "Date": "",
-        "Volume": "",
+        "volume": "",
         "Trend": "",
         "MA-Signal": "",
         "LTP": "",
@@ -123,7 +123,7 @@ def backtest(
     backTestedStock["Consol."] = screenedDict["Consol."]
     backTestedStock["Breakout"] = screenedDict["Breakout"]
     backTestedStock["MA-Signal"] = screenedDict["MA-Signal"]
-    backTestedStock["Volume"] = screenedDict["Volume"]
+    backTestedStock["volume"] = screenedDict["volume"]
     backTestedStock["LTP"] = screenedDict["LTP"]
     backTestedStock["52Wk-H"] = screenedDict["52Wk-H"]
     backTestedStock["52Wk-L"] = screenedDict["52Wk-L"]
@@ -136,7 +136,7 @@ def backtest(
             backTestedStock[f"{abs(prd)}-Pd"] = ""
             backTestedStock[f"LTP{prd}"] = ""
             backTestedStock[f"Growth{prd}"] = ""
-            rolling_pct = data["Close"].pct_change(periods=prd) * 100
+            rolling_pct = data["close"].pct_change(periods=prd) * 100
             pct_change = rolling_pct.iloc[prd]
             if not sellSignal:
                 colored_pct = colorText.GREEN if pct_change >= 0 else colorText.FAIL
@@ -217,7 +217,7 @@ def backtestSummary(df):
                 else:
                     summary[
                         col
-                    ] = f"{Utility.tools.formattedBacktestOutput(overAllPeriodPrediction)} of ({col_positives+col_negatives})"
+                    ] = f"{ConsoleUtility.PKConsoleTools.formattedBacktestOutput(overAllPeriodPrediction)} of ({col_positives+col_negatives})"
         overAllRowPrediction = (
             group_positives * 100 / (group_positives + group_negatives)
         )
@@ -226,7 +226,7 @@ def backtestSummary(df):
         else:
             summary[
                 "Overall"
-            ] = f"{Utility.tools.formattedBacktestOutput(overAllRowPrediction)} of ({group_positives+group_negatives})"
+            ] = f"{ConsoleUtility.PKConsoleTools.formattedBacktestOutput(overAllRowPrediction)} of ({group_positives+group_negatives})"
         summaryList.append(summary)
         summary = {}
         net_positives += group_positives
@@ -242,13 +242,13 @@ def backtestSummary(df):
         else:
             summary[
                 col
-            ] = f"{Utility.tools.formattedBacktestOutput((col_positives*100/(col_positives+col_negatives)))} of ({col_positives+col_negatives})"
+            ] = f"{ConsoleUtility.PKConsoleTools.formattedBacktestOutput((col_positives*100/(col_positives+col_negatives)))} of ({col_positives+col_negatives})"
     if net_positives + net_negatives == 0:
         summary["Overall"] = "-"
     else:
         summary[
             "Overall"
-        ] = f"{Utility.tools.formattedBacktestOutput(net_positives*100/(net_positives+net_negatives))} of ({net_positives+net_negatives})"
+        ] = f"{ConsoleUtility.PKConsoleTools.formattedBacktestOutput(net_positives*100/(net_positives+net_negatives))} of ({net_positives+net_negatives})"
     summaryList.append(summary)
     summary_df = pd.DataFrame(summaryList, columns=summary.keys())
     return summary_df
