@@ -39,6 +39,7 @@ class CandlePatterns:
         "Dragonfly Doji",
         "Supply Drought",
         "Demand Rise",
+        "Cup and Handle",
     ]
     reversalPatternsBearish = [
         "Evening Star",
@@ -65,177 +66,185 @@ class CandlePatterns:
     #@measure_time
     # Find candle-stick patterns
     # Arrange if statements with max priority from top to bottom
-    def findPattern(self, data, dict, saveDict):
-        data = data.head(4)
+    def findPattern(self, processedData, dict, saveDict,filterPattern=None):
+        data = processedData.head(4)
         data = data[::-1]
         hasCandleStickPattern = False
         if "Pattern" not in saveDict.keys():
             saveDict["Pattern"] = ""
             dict["Pattern"] = ""
-        # Only 'doji' and 'inside' is internally implemented by pandas_ta.
+        # Only 'doji' and 'inside' is internally implemented by pandas_ta_classic.
         # Otherwise, for the rest of the candle patterns, they also need
         # TA-Lib.
-        check = pktalib.CDLDOJI(data["Open"], data["High"], data["Low"], data["Close"])
+        check = pktalib.CDLDOJI(data["open"], data["high"], data["low"], data["close"])
         if check is not None and check.tail(1).item() != 0:
-            dict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + colorText.BOLD + f"Doji" + colorText.END 
+            dict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + colorText.GREEN + f"Doji" + colorText.END 
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Doji"
             hasCandleStickPattern = True
 
         check = pktalib.CDLMORNINGSTAR(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.GREEN + f"Morning Star" + colorText.END 
+                colorText.GREEN + f"Morning Star" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Morning Star"
             hasCandleStickPattern = True
+        
+        check = pktalib.CDLCUPANDHANDLE(
+            processedData["open"], processedData["high"], processedData["low"], processedData["close"]
+        )
+        if check:
+            dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
+                colorText.GREEN + f"Cup and Handle" + colorText.END 
+            )
+            saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Cup and Handle"
+            hasCandleStickPattern = True
 
         check = pktalib.CDLMORNINGDOJISTAR(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.GREEN + f"Morning Doji Star" + colorText.END 
+                colorText.GREEN + f"Morning Doji Star" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Morning Doji Star"
             hasCandleStickPattern = True
 
         check = pktalib.CDLEVENINGSTAR(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.FAIL + f"Evening Star" + colorText.END 
+                colorText.FAIL + f"Evening Star" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Evening Star"
             hasCandleStickPattern = True
 
         check = pktalib.CDLEVENINGDOJISTAR(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.FAIL + f"Evening Doji Star" + colorText.END 
+                colorText.FAIL + f"Evening Doji Star" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Evening Doji Star"
             hasCandleStickPattern = True
 
         check = pktalib.CDLLADDERBOTTOM(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             if check.tail(1).item() > 0:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD + colorText.GREEN + f"Bullish Ladder Bottom" + colorText.END 
+                    colorText.GREEN + f"Bullish Ladder Bottom" + colorText.END 
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Bullish Ladder Bottom"
             else:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD + colorText.FAIL + f"Bearish Ladder Bottom" + colorText.END 
+                    colorText.FAIL + f"Bearish Ladder Bottom" + colorText.END 
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Bearish Ladder Bottom"
             hasCandleStickPattern = True
 
         check = pktalib.CDL3LINESTRIKE(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             if check.tail(1).item() > 0:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD + colorText.GREEN + f"3 Line Strike" + colorText.END 
+                    colorText.GREEN + f"3 Line Strike" + colorText.END 
                 )
             else:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD + colorText.FAIL + f"3 Line Strike" + colorText.END 
+                    colorText.FAIL + f"3 Line Strike" + colorText.END 
                 )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"3 Line Strike"
             hasCandleStickPattern = True
 
         check = pktalib.CDL3BLACKCROWS(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.FAIL + f"3 Black Crows" + colorText.END 
+                colorText.FAIL + f"3 Black Crows" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"3 Black Crows"
             hasCandleStickPattern = True
 
         check = pktalib.CDL3INSIDE(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             if check.tail(1).item() > 0:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD + colorText.GREEN + f"3 Inside Up" + colorText.END 
+                    colorText.GREEN + f"3 Inside Up" + colorText.END 
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"3 Outside Up"
             else:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD + colorText.FAIL + f"3 Inside Down" + colorText.END 
+                    colorText.FAIL + f"3 Inside Down" + colorText.END 
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"3 Inside Down"
             hasCandleStickPattern = True
 
         check = pktalib.CDL3OUTSIDE(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             if check.tail(1).item() > 0:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD + colorText.GREEN + f"3 Outside Up" + colorText.END 
+                    colorText.GREEN + f"3 Outside Up" + colorText.END 
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"3 Outside Up"
             else:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD + colorText.FAIL + f"3 Outside Down" + colorText.END 
+                    colorText.FAIL + f"3 Outside Down" + colorText.END 
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"3 Outside Down"
             hasCandleStickPattern = True
 
         check = pktalib.CDL3WHITESOLDIERS(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.GREEN + f"3 White Soldiers" + colorText.END 
+                colorText.GREEN + f"3 White Soldiers" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"3 White Soldiers"
             hasCandleStickPattern = True
 
         check = pktalib.CDLHARAMI(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             if check.tail(1).item() > 0:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD + colorText.GREEN + f"Bullish Harami" + colorText.END 
+                    colorText.GREEN + f"Bullish Harami" + colorText.END 
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Bullish Harami"
             else:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD + colorText.FAIL + f"Bearish Harami" + colorText.END 
+                    colorText.FAIL + f"Bearish Harami" + colorText.END 
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Bearish Harami"
             hasCandleStickPattern = True
 
         check = pktalib.CDLHARAMICROSS(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             if check.tail(1).item() > 0:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD
-                    + colorText.GREEN
+                    colorText.GREEN
                     + f"Bullish Harami Cross" 
                     + colorText.END
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Bullish Harami Cross"
             else:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD
-                    + colorText.FAIL
+                    colorText.FAIL
                     + f"Bearish Harami Cross" 
                     + colorText.END
                 )
@@ -243,105 +252,102 @@ class CandlePatterns:
             hasCandleStickPattern = True
 
         check = pktalib.CDLMARUBOZU(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             if check.tail(1).item() > 0:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD
-                    + colorText.GREEN
+                    colorText.GREEN
                     + f"Bullish Marubozu" 
                     + colorText.END
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Bullish Marubozu"
             else:
                 dict["Pattern"] = (
-                    colorText.BOLD + colorText.FAIL + f"Bearish Marubozu" + colorText.END 
+                    colorText.FAIL + f"Bearish Marubozu" + colorText.END 
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Bearish Marubozu"
             hasCandleStickPattern = True
 
         check = pktalib.CDLHANGINGMAN(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.FAIL + f"Hanging Man" + colorText.END 
+                colorText.FAIL + f"Hanging Man" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Hanging Man"
             hasCandleStickPattern = True
 
         check = pktalib.CDLHAMMER(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.GREEN + f"Hammer" + colorText.END 
+                colorText.GREEN + f"Hammer" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Hammer"
             hasCandleStickPattern = True
 
         check = pktalib.CDLINVERTEDHAMMER(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.GREEN + f"Inverted Hammer" + colorText.END 
+                colorText.GREEN + f"Inverted Hammer" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Inverted Hammer"
             hasCandleStickPattern = True
 
         check = pktalib.CDLSHOOTINGSTAR(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.FAIL + f"Shooting Star" + colorText.END 
+                colorText.FAIL + f"Shooting Star" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Shooting Star"
             hasCandleStickPattern = True
 
         check = pktalib.CDLDRAGONFLYDOJI(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.GREEN + f"Dragonfly Doji" + colorText.END 
+                colorText.GREEN + f"Dragonfly Doji" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Dragonfly Doji"
             hasCandleStickPattern = True
 
         check = pktalib.CDLGRAVESTONEDOJI(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                colorText.BOLD + colorText.FAIL + f"Gravestone Doji" + colorText.END 
+                colorText.FAIL + f"Gravestone Doji" + colorText.END 
             )
             saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Gravestone Doji"
             hasCandleStickPattern = True
 
         check = pktalib.CDLENGULFING(
-            data["Open"], data["High"], data["Low"], data["Close"]
+            data["open"], data["high"], data["low"], data["close"]
         )
         if check is not None and check.tail(1).item() != 0:
             if check.tail(1).item() > 0:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD
-                    + colorText.GREEN
+                    colorText.GREEN
                     + f"Bullish Engulfing" 
                     + colorText.END
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Bullish Engulfing"
             else:
                 dict["Pattern"] = (self.findCurrentSavedValue(dict,saveDict,"Pattern")[0] + 
-                    colorText.BOLD
-                    + colorText.FAIL
+                    colorText.FAIL
                     + f"Bearish Engulfing" 
                     + colorText.END
                 )
                 saveDict["Pattern"] = self.findCurrentSavedValue(dict,saveDict,"Pattern")[1] +  f"Bearish Engulfing"
             hasCandleStickPattern = True
         if hasCandleStickPattern:
-            return True
+            return filterPattern in saveDict["Pattern"] if filterPattern is not None else hasCandleStickPattern
         return False
